package ch.uzh.ifi.ddis.ifp.esper.cassandra;

/*
 * #%L
 * Cassandra for Esper
 * %%
 * Copyright (C) 2013 University of Zurich
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * #L%
 */

import java.util.Iterator;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.datastax.driver.core.Cluster;
import com.datastax.driver.core.PreparedStatement;
import com.datastax.driver.core.Session;
import com.espertech.esper.client.EventBean;
import com.espertech.esper.client.hook.VirtualDataWindow;
import com.espertech.esper.client.hook.VirtualDataWindowContext;
import com.espertech.esper.client.hook.VirtualDataWindowEvent;
import com.espertech.esper.client.hook.VirtualDataWindowLookup;
import com.espertech.esper.client.hook.VirtualDataWindowLookupContext;

/**
 * The virtual data window for Esper which stores results from a lookup to
 * Cassandra.
 * 
 * @author Thomas Scharrenbach
 * @version 0.0.1
 * @since 0.0.1
 * 
 */
public class CassandraVirtualDataWindow implements VirtualDataWindow {

	private static final Logger _log = LoggerFactory
			.getLogger(CassandraVirtualDataWindow.class);

	//
	//
	//

	private final Cluster _cluster;

	private Session _session;

	private final VirtualDataWindowContext _context;

	private PreparedStatement _preparedStatement;

	//
	//
	//

	/**
	 * Simple copy constructor.
	 * 
	 * @param cluster
	 *            the reference to the Cassandra cluster.
	 * @param context
	 *            the Esper context.
	 * @param preparedStatementString
	 */
	CassandraVirtualDataWindow(Cluster cluster,
			VirtualDataWindowContext context, String keyspace, String preparedStatementString) {
		_cluster = cluster;
		_context = context;
		_session = _cluster.connect(keyspace);
		_preparedStatement = _session.prepare(preparedStatementString);
	}

	//
	//
	//

	/**
	 * @return a new {@link CassandraVirtualDataWindowLookup} with this window's
	 *         Cassandra session and Esper context.
	 */
	@Override
	public VirtualDataWindowLookup getLookup(VirtualDataWindowLookupContext desc) {
		return new CassandraVirtualDataWindowLookup(_session, _context,
				_preparedStatement);
	}

	/**
	 * Currently, this method performs nothing.
	 */
	@Override
	public void handleEvent(VirtualDataWindowEvent theEvent) {
	}

	/**
	 * <p>
	 * Currently simply posts into the insert and remove stream what is
	 * received.
	 * </p>
	 */
	@Override
	public void update(EventBean[] newData, EventBean[] oldData) {
		_context.getOutputStream().update(newData, oldData);
	}

	/**
	 * Shutdown the Cassandra session.
	 */
	@Override
	public void destroy() {
		if (_session != null) {
			try {
				_log.info("Started shutting down session to Cassandra cluster...");
				_session.shutdown();
				_log.info("Finished shutting down session to Cassandra cluster.");
			}
			//
			catch (Exception e) {
				_log.error("Error shutting down session to Cassandra cluster!",
						e);
			}
		}
	}

	/**
	 * Currently returns an empty iterator. This might change in the future.
	 */
	@Override
	public Iterator<EventBean> iterator() {
		return java.util.Collections.<EventBean> emptyList().iterator();
	}

	//
	//
	//

}
