package ch.uzh.ifi.ddis.ifp.esper.cassandra;

/*
 * #%L
 * Cassandra for Esper
 * %%
 * Copyright (C) 2013 University of Zurich
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * #L%
 */

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.datastax.driver.core.Cluster;
import com.espertech.esper.client.hook.VirtualDataWindow;
import com.espertech.esper.client.hook.VirtualDataWindowContext;
import com.espertech.esper.client.hook.VirtualDataWindowFactory;
import com.espertech.esper.client.hook.VirtualDataWindowFactoryContext;

/**
 * <p>
 * Factory for creating {@link CassandraVirtualDataWindow} objects.
 * </p>
 * 
 * @author Thomas Scharrenbach
 * @version 0.0.1
 * @since 0.0.1
 * 
 */
public class CassandraVirtualDataWindowFactory implements
		VirtualDataWindowFactory {

	private static final Logger _log = LoggerFactory
			.getLogger(CassandraVirtualDataWindowFactory.class);

	private static final String CASSANDRA_HOST = "host";

	private static final String CASSANDRA_KEYSPACE = "keyspace";

	private static final String CASSANDRA_STATEMENT = "stmt";

	private static final String CASSANDRA_STATEMENT_KEYS = "stmt-keys";

	//
	//
	//

	private Cluster _cluster;

	private String _cassandraPreparedStatement;

	private String _keyspace;

	//
	//
	//

	public CassandraVirtualDataWindowFactory() {
		return;
	}

	//
	//
	//

	/**
	 * @throws java.lang.RuntimeException
	 *             In case the connection to Cassandra fails.
	 */
	@Override
	public void initialize(VirtualDataWindowFactoryContext factoryContext) {
		final Object[] windowParameters = factoryContext.getParameters();

		if (windowParameters.length < 1) {
			throw new RuntimeException(String.format(
					"Cassandra window has at least one parameter. "
							+ "Found %s", windowParameters.length));
		}

		String cassandraHost = null;
		final Map<String, String> esperEventBeanKeysToCassandraStatementKeys = new LinkedHashMap<String, String>();

		_log.info("Started parsing Cassandra configuration...");
		final JSONParser parser = new JSONParser();
		try {
			final JSONObject cassandraConfig = (JSONObject) parser
					.parse(windowParameters[0].toString());
			Object tmp = null;
			_log.debug("Cassandra configuration: {}", cassandraConfig);

			//
			try {
				tmp = cassandraConfig.get(CASSANDRA_HOST).toString();
				cassandraHost = tmp.toString();
			} catch (Exception e) {
				throw new IllegalArgumentException(String.format(
						"Illegal argument for %s with value %s !",
						CASSANDRA_HOST, tmp), e);
			}

			//
			try {
				tmp = cassandraConfig.get(CASSANDRA_KEYSPACE).toString();
				_keyspace = tmp.toString();
			} catch (Exception e) {
				throw new IllegalArgumentException(String.format(
						"Illegal argument for %s with value %s !",
						CASSANDRA_KEYSPACE, tmp), e);
			}

			//
			try {
				tmp = cassandraConfig.get(CASSANDRA_STATEMENT).toString();
				_cassandraPreparedStatement = tmp.toString();
			} catch (Exception e) {
				throw new IllegalArgumentException(String.format(
						"Illegal argument for %s with value %s !",
						CASSANDRA_STATEMENT, tmp), e);
			}

			//
			try {
				tmp = cassandraConfig.get(CASSANDRA_STATEMENT_KEYS).toString();
				final JSONArray cassandraKeysTmp = (JSONArray) parser.parse(tmp
						.toString());
				for (Object mappingObject : cassandraKeysTmp) {
					final JSONObject mapping = (JSONObject) mappingObject;
					// TODO implement verification
					esperEventBeanKeysToCassandraStatementKeys.put(mapping
							.keySet().iterator().next().toString(), mapping
							.values().iterator().next().toString());
				}

			} catch (Exception e) {
				throw new IllegalArgumentException(String.format(
						"Illegal argument for %s with value %s !",
						CASSANDRA_STATEMENT_KEYS, tmp), e);
			}
		} catch (Exception pe) {
			_log.error("Error parsing Cassandra configuration!");
			throw new RuntimeException(pe);
		}
		_log.info("Finished parsing Cassandra configuration.");

		try {
			connect(cassandraHost);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}

	}

	@Override
	public VirtualDataWindow create(VirtualDataWindowContext context) {
		if (_cluster == null) {
			throw new RuntimeException(
					"Cassandra cluster has not yet been initialized!");
		}
		return new CassandraVirtualDataWindow(_cluster, context, _keyspace,
				_cassandraPreparedStatement);
	}

	/**
	 * Shutsdown the connection to the Cassandra cluster.
	 */
	@Override
	public void destroyAllContextPartitions() {
		close();
	}

	@Override
	public Set<String> getUniqueKeyPropertyNames() {
		// TODO Auto-generated method stub
		return null;
	}

	//
	//
	//

	/**
	 * Establish connection to Cassandra cluster.
	 * 
	 * @param node
	 *            address of the cluster.
	 */
	protected void connect(String node) throws Exception {
		try {
			_log.info("Started connecting to cluster: {} ...", node);
			_cluster = Cluster.builder().addContactPoint(node).build();

			_log.info("Finished connecting to cluster: {} .", node);
		} catch (Exception e) {
			_log.error("Error connecting to cluster: {} !", node);

			final String errorMessage = String.format(
					"Could not connect to Cassandra cluster!", node);
			throw new RuntimeException(errorMessage, e);
		}
	}

	/**
	 * Disconnect from Cassandra cluster.
	 */
	protected void close() {
		try {
			_log.info("Started shutting down connection to Cassandra cluster...");
			_cluster.shutdown();
			_log.info("Finished shutting down connection to Cassandra cluster.");
		}
		//
		catch (Exception e) {
			_log.error("Error shutting down connection to Cassandra cluster!",
					e);
		}
	}

}
