package ch.uzh.ifi.ddis.ifp.esper.cassandra;

/*
 * #%L
 * Cassandra for Esper
 * %%
 * Copyright (C) 2013 University of Zurich
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * #L%
 */

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.datastax.driver.core.BoundStatement;
import com.datastax.driver.core.PreparedStatement;
import com.datastax.driver.core.ResultSet;
import com.datastax.driver.core.Row;
import com.datastax.driver.core.Session;
import com.espertech.esper.client.EventBean;
import com.espertech.esper.client.hook.VirtualDataWindowContext;
import com.espertech.esper.client.hook.VirtualDataWindowLookup;

/**
 * <p>
 * Esper lookup to Cassandra.
 * </p>
 * <p>
 * A {@link CassandraVirtualDataWindowLookup} performs queries against a
 * Cassandra cluster and inserts the values into a <a href=
 * "http://esper.codehaus.org/esper-4.9.0/doc/reference/en-US/html_single/#extension-virtualdw"
 * >virtual Esper data window</a>.
 * </p>
 * 
 * 
 * @author Thomas Scharrenbach
 * @version 0.0.1
 * @since 0.0.1
 * 
 */
public class CassandraVirtualDataWindowLookup implements
		VirtualDataWindowLookup {

	private final Session _session;

	private final VirtualDataWindowContext _context;

	private PreparedStatement _statement;

	//
	//
	//

	/**
	 * <p>
	 * Creates a new lookup object that performs queries against a Cassandra
	 * cluster and inserts the values into a virtual Esper data window.
	 * </p>
	 * 
	 * @param session
	 *            the Cassandra session for performing queries against a
	 *            Cassandra cluster.
	 * @param context
	 *            the Esper context for creating EventBean objects.
	 * @param preparedStatement
	 *            the prepared Esper statement for the lookup.
	 */
	CassandraVirtualDataWindowLookup(Session session,
			VirtualDataWindowContext context,
			PreparedStatement preparedStatement) {
		_session = session;
		_context = context;
		_statement = preparedStatement;
	}

	//
	//
	//

	@Override
	public Set<EventBean> lookup(Object[] keys, EventBean[] eventsPerStream) {
		synchronized (_statement) {

			final BoundStatement boundStatement = _statement.bind(keys);

			final ResultSet cassandraResultList = _session
					.execute(boundStatement);

			final Set<EventBean> esperResultSet = new HashSet<EventBean>();
			final Iterator<Row> it = cassandraResultList.iterator();
			while (it.hasNext()) {
				final Map<String, Object> eventData = new HashMap<String, Object>();
				final Row cassandraRow = it.next();

				// TODO: extract values from row into event bean.

				final EventBean event = _context.getEventFactory().wrap(
						eventData);
				esperResultSet.add(event);
			}

			return esperResultSet;
		}
	}

}
